@@ -9,35 +9,36 @@
 #include <config.h>
 #include <string.h>
 #include <signal.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <libbonobo.h>
 #include <gdk/gdkx.h>
 
-#include "panel-include.h"
-#include "gnome-panel.h"
-#include "scroll-menu.h"
+#include "basep-widget.h"
+#include "button-widget.h"
+#include "drawer.h"
+#include "extern.h"
+#include "launcher.h"
+#include "menu-properties.h"
+#include "menu-util.h"
+#include "panel_config.h"
+#include "panel_config_global.h"
+#include "session.h"
+#include "status.h"
+#include "swallow.h"
 
 #define SMALL_ICON_SIZE 20
 
-#define APPLET_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
-			   GDK_BUTTON_RELEASE_MASK |		\
-			   GDK_POINTER_MOTION_MASK |		\
-			   GDK_POINTER_MOTION_HINT_MASK)
-
 extern GSList *panels;
 
 GSList *applets = NULL;
-GSList *applets_last = NULL;
 int applet_count = 0;
 
 /*config sync stuff*/
-extern int config_sync_timeout;
 extern int applets_to_sync;
-extern int panels_to_sync;
 extern int need_complete_save;
 
 extern gboolean commie_mode;
 extern GlobalConfig global_config;
-extern PanelWidget *current_panel;
 
 static void
 move_applet_callback (GtkWidget *widget, AppletInfo *info)
@@ -47,7 +48,7 @@ move_applet_callback (GtkWidget *widget, AppletInfo *info)
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (info->widget != NULL);
 	g_return_if_fail (info->widget->parent != NULL);
-	g_return_if_fail (IS_PANEL_WIDGET (info->widget->parent));
+	g_return_if_fail (PANEL_IS_WIDGET (info->widget->parent));
 
 	panel = PANEL_WIDGET (info->widget->parent);
 
@@ -57,7 +58,7 @@ move_applet_callback (GtkWidget *widget, AppletInfo *info)
 
 /*destroy widgets and call the above cleanup function*/
 void
-panel_clean_applet (AppletInfo *info)
+panel_applet_clean (AppletInfo *info)
 {
 	g_return_if_fail (info != NULL);
 
@@ -77,15 +78,18 @@ static gboolean
 kill_applet_in_idle(gpointer data)
 {
 	AppletInfo *info = data;
-	if (info->type == APPLET_EXTERN) {
-		Extern *ext = info->data;
-		extern_save_last_position (ext, TRUE /* sync */);
-		ext->clean_remove = TRUE;
-	} else if (info->type == APPLET_SWALLOW) {
+
+	if (info->type == APPLET_EXTERN)
+		extern_save_last_position ((Extern)info->data, TRUE);
+
+	else if (info->type == APPLET_SWALLOW) {
 		Swallow *swallow = info->data;
+
 		swallow->clean_remove = TRUE;
 	}
-	panel_clean_applet (info);
+
+	panel_applet_clean (info);
+
 	return FALSE;
 }
 
@@ -104,47 +108,42 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 
 	switch(menu->info->type) {
 	case APPLET_EXTERN:
-		{
-			CORBA_Environment ev;
-			Extern *ext = menu->info->data;
-			g_assert(ext);
-			g_assert(ext->applet);
-
-			CORBA_exception_init(&ev);
-			GNOME_Applet_do_callback(ext->applet, menu->name, &ev);
-			if(ev._major)
-				panel_clean_applet(ext->info);
-			CORBA_exception_free(&ev);
-			break;
-		}
+		extern_handle_do_callback ((Extern)menu->info->data,
+					   menu->name);
+		break;
 	case APPLET_LAUNCHER:
-		if(strcmp(menu->name,"properties")==0)
-			launcher_properties(menu->info->data);
-		else if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("launchers.html");
-		else if (strcmp (menu->name, "help_on_app") == 0) {
+		if (strcmp (menu->name, "properties") == 0) {
+			launcher_properties (menu->info->data);
+		} else if (strcmp (menu->name, "help") == 0) {
+			panel_show_help ("launchers", NULL);
+		} else if (strcmp (menu->name, "help_on_app") == 0) {
 			Launcher * launcher = menu->info->data;
-			if (launcher->dentry != NULL) {
-				char *path = panel_gnome_kde_help_path
-					(launcher->dentry->docpath);
+			if (launcher->ditem != NULL) {
+				const char *docpath =
+					gnome_desktop_item_get_string
+					    (launcher->ditem, "DocPath");
+				char *path =
+					panel_gnome_kde_help_path (docpath);
 				if (path != NULL) {
-					gnome_url_show (path);
+					gnome_url_show (path, NULL);
+					/* FIXME: handle errors */
 					g_free (path);
 				}
 			}
 		}
 		break;
 	case APPLET_DRAWER: 
-		if(strcmp(menu->name,"properties")==0) {
+		if (strcmp (menu->name, "properties")==0) {
 			Drawer *drawer = menu->info->data;
 			g_assert(drawer);
 			panel_config(drawer->drawer);
-		} else if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("drawers.html");
+		} else if (strcmp (menu->name, "help") == 0) {
+			panel_show_help ("drawers", NULL);
+		}
 		break;
 	case APPLET_SWALLOW:
  		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects.html#SWALLOWEDAPP");
+			panel_show_help ("specialobjects", "SWALLOWEDAPP");
 #if 0
 		if(strcmp(menu->name,"properties")==0) {
 			Swallow *swallow = info->data;
@@ -158,16 +157,15 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			menu_properties(menu->info->data);
 		else if(strcmp(menu->name,"edit_menus")==0) {
 			char *tmp;
-			if((tmp = panel_is_program_in_path("gmenu")))  {
+			if((tmp = g_find_program_in_path ("gmenu")))  {
 				gnome_execute_async (g_get_home_dir (), 1, &tmp);
 				g_free(tmp);
 			}
 		} else if (strcmp (menu->name, "help") == 0) {
 			Menu *menu2 = menu->info->data;
 			char *page;
-			page = (menu2->path && strcmp (menu2->path,"."))
-				? "menus.html" : "mainmenu.html";
-			panel_show_help (page);
+			page = menu2->main_menu ? "mainmenu" : "menus";
+			panel_show_help (page, NULL);
 		}
 		break;
 	case APPLET_LOCK: {
@@ -185,7 +183,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		char *command = NULL;
 		gboolean freeit = FALSE;
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects.html#LOCKBUTTON");
+			panel_show_help ("specialobjects", "LOCKBUTTON");
 		else if (!strcmp (menu->name, "restart")) {
 			command = "xscreensaver-command -exit ; xscreensaver &";
 		} else if (!strcmp (menu->name, "prefs")) {
@@ -203,15 +201,15 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 	}
 	case APPLET_LOGOUT:
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects.html#LOGOUTBUTTON");
+			panel_show_help ("specialobjects", "LOGOUTBUTTON");
 		break;
 	case APPLET_STATUS:
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects.html#STATUSDOC");
+			panel_show_help ("specialobjects", "STATUSDOC");
 		break;
 	case APPLET_RUN:
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects.html#RUNBUTTON");
+			panel_show_help ("specialobjects", "RUNBUTTON");
 		break;
 	default: break;
 	}
@@ -223,18 +221,20 @@ applet_menu_deactivate(GtkWidget *w, AppletInfo *info)
 	GtkWidget *panel = get_panel_parent(info->widget);
 	info->menu_age = 0;
 	
-	if(IS_BASEP_WIDGET(panel))
+	if(BASEP_IS_WIDGET(panel))
 		BASEP_WIDGET(panel)->autohide_inhibit = FALSE;
 }
 
 AppletUserMenu *
-applet_get_callback (GList *user_menu, const char *name)
+panel_applet_get_callback (GList      *user_menu,
+			   const char *name)
 {
-	GList *li;
+	GList *l;
 
-	for (li = user_menu; li != NULL; li = li->next) {
-		AppletUserMenu *menu = li->data;
-		if (strcmp (menu->name, name) == 0)
+	for (l = user_menu; l; l = l->next) {
+		AppletUserMenu *menu = l->data;
+
+		if (!strcmp (menu->name, name))
 			return menu;
 	}
 
@@ -242,16 +242,16 @@ applet_get_callback (GList *user_menu, const char *name)
 }
 
 void
-applet_add_callback(AppletInfo *info,
-		    const char *callback_name,
-		    const char *stock_item,
-		    const char *menuitem_text)
+panel_applet_add_callback (AppletInfo *info,
+			   const char *callback_name,
+			   const char *stock_item,
+			   const char *menuitem_text)
 {
 	AppletUserMenu *menu;
 
 	g_return_if_fail (info != NULL);
 	
-	menu = applet_get_callback (info->user_menu, callback_name);
+	menu = panel_applet_get_callback (info->user_menu, callback_name);
 	if (menu == NULL) {
 		menu = g_new0 (AppletUserMenu, 1);
 		menu->name = g_strdup (callback_name);
@@ -286,13 +286,14 @@ applet_add_callback(AppletInfo *info,
 }
 
 void
-applet_remove_callback(AppletInfo *info, const char *callback_name)
+panel_applet_remove_callback (AppletInfo *info,
+			      const char *callback_name)
 {
 	AppletUserMenu *menu;
 
 	g_return_if_fail (info != NULL);
 	
-	menu = applet_get_callback (info->user_menu, callback_name);
+	menu = panel_applet_get_callback (info->user_menu, callback_name);
 	if (menu != NULL) {
 		info->user_menu = g_list_remove (info->user_menu, menu);
 		g_free(menu->name);
@@ -320,13 +321,15 @@ applet_remove_callback(AppletInfo *info, const char *callback_name)
 }
 
 void
-applet_callback_set_sensitive(AppletInfo *info, const char *callback_name, int sensitive)
+panel_applet_callback_set_sensitive (AppletInfo  *info,
+				     const gchar *callback_name,
+				     gint         sensitive)
 {
 	AppletUserMenu *menu;
 
 	g_return_if_fail(info != NULL);
 	
-	menu = applet_get_callback(info->user_menu, callback_name);
+	menu = panel_applet_get_callback(info->user_menu, callback_name);
 	if (menu != NULL)
 		menu->sensitive = sensitive;
 	else
@@ -352,13 +355,13 @@ setup_an_item(AppletUserMenu *menu,
 	      int is_submenu)
 {
 	menu->menuitem = gtk_menu_item_new ();
-	gtk_signal_connect (GTK_OBJECT (menu->menuitem), "destroy",
-			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-			    &menu->menuitem);
+	g_signal_connect (G_OBJECT (menu->menuitem), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed),
+			  &menu->menuitem);
 	if(menu->stock_item && *(menu->stock_item))
 		setup_menuitem (menu->menuitem,
-				gnome_stock_pixmap_widget(submenu,
-							  menu->stock_item),
+				gtk_image_new_from_stock (menu->stock_item,
+							  GTK_ICON_SIZE_MENU),
 				menu->text);
 	else
 		setup_menuitem (menu->menuitem,
@@ -366,27 +369,27 @@ setup_an_item(AppletUserMenu *menu,
 				menu->text);
 
 	if(submenu)
-		gtk_menu_append (GTK_MENU (submenu), menu->menuitem);
+		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu->menuitem);
 
 	/*if an item not a submenu*/
 	if(!is_submenu) {
-		gtk_signal_connect(GTK_OBJECT(menu->menuitem), "activate",
-				   (GtkSignalFunc) applet_callback_callback,
+		g_signal_connect(G_OBJECT(menu->menuitem), "activate",
+				 G_CALLBACK (applet_callback_callback),
 				   menu);
-		gtk_signal_connect(GTK_OBJECT (submenu), "destroy",
-				   GTK_SIGNAL_FUNC (gtk_widget_destroyed),
-				   &menu->submenu);
+		g_signal_connect(G_OBJECT (submenu), "destroy",
+				 G_CALLBACK (gtk_widget_destroyed),
+				 &menu->submenu);
 	/* if the item is a submenu and doesn't have it's menu
 	   created yet*/
 	} else if(!menu->submenu) {
-		menu->submenu = hack_scroll_menu_new();
+		menu->submenu = gtk_menu_new ();
 	}
 
 	if(menu->submenu) {
 		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu->menuitem),
 					  menu->submenu);
-		gtk_signal_connect (GTK_OBJECT (menu->submenu), "destroy",
-				    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
+		g_signal_connect (G_OBJECT (menu->submenu), "destroy",
+				    G_CALLBACK (gtk_widget_destroyed),
 				    &menu->submenu);
 	}
 	
@@ -424,7 +427,7 @@ add_to_submenus (AppletInfo *info,
 	p++;
 	
 	t = g_strconcat (path, n, "/", NULL);
-	s_menu = applet_get_callback (user_menu, t);
+	s_menu = panel_applet_get_callback (user_menu, t);
 	/*the user did not give us this sub menu, whoops, will create an empty
 	  one then*/
 	if (s_menu == NULL) {
@@ -441,7 +444,7 @@ add_to_submenus (AppletInfo *info,
 	}
 	
 	if (s_menu->submenu == NULL) {
-		s_menu->submenu = hack_scroll_menu_new();
+		s_menu->submenu = gtk_menu_new ();
 		/*a more elegant way to do this should be done
 		  when I don't want to go to sleep */
 		if (s_menu->menuitem != NULL) {
@@ -458,161 +461,199 @@ add_to_submenus (AppletInfo *info,
 	g_free(n);
 }
 
-void
-create_applet_menu (AppletInfo *info, gboolean is_basep)
+static GtkWidget *
+applet_setup_panel_menu (gboolean is_basep)
 {
-	GtkWidget *menuitem, *panel_menu;
-	GList *user_menu = info->user_menu;
-	gchar *pixmap;
+	GtkWidget *menuitem;
+	GtkWidget *panel_menu;
+	gchar     *pixmap_path;
 
-	info->menu = hack_scroll_menu_new ();
-
-	if ( ! commie_mode) {
-		menuitem = gtk_menu_item_new();
-		setup_menuitem(menuitem,
-			       gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REMOVE),
-			       _("Remove from panel"));
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-				   (GtkSignalFunc) remove_applet_callback,
-				   info);
-		gtk_menu_append(GTK_MENU(info->menu), menuitem);
-
-		menuitem = gtk_menu_item_new();
-		setup_menuitem(menuitem,NULL,_("Move"));
-		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
-				   (GtkSignalFunc) move_applet_callback,
-				   info);
-		gtk_menu_append(GTK_MENU(info->menu), menuitem);
-	}
-
-	panel_menu = hack_scroll_menu_new();
-	make_panel_submenu (panel_menu, TRUE, is_basep);
 	menuitem = gtk_menu_item_new ();
 
-	pixmap = gnome_pixmap_file ("gnome-panel.png");
-	if (pixmap == NULL) {
-		g_message (_("Cannot find pixmap file %s"), "gnome-panel.png");
-		setup_menuitem (menuitem, NULL, _("Panel"));
-	} else {
+	pixmap_path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+						 "gnome-panel.png", TRUE, NULL);
+	if (pixmap_path) {
 		setup_menuitem (menuitem, 
-				gnome_stock_pixmap_widget_at_size (NULL, pixmap,
-								   SMALL_ICON_SIZE,
-								   SMALL_ICON_SIZE),
+				gtk_image_new_from_file (pixmap_path),
 				_("Panel"));
-		g_free (pixmap);
+
+		g_free (pixmap_path);
+	} else {
+		g_message (_("Cannot find pixmap file %s"), "gnome-panel.png");
+
+		setup_menuitem (menuitem, NULL, _("Panel"));
 	}
-	gtk_menu_append (GTK_MENU (info->menu), menuitem);
+
+	panel_menu = gtk_menu_new ();
+
+	make_panel_submenu (panel_menu, TRUE, is_basep);
+
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), panel_menu);
 
-	if(user_menu) {
+	return menuitem;
+}
+
+void
+panel_applet_create_menu (AppletInfo *info,
+			  gboolean    is_basep)
+{
+	GtkWidget *menuitem;
+	GList     *l;
+
+	info->menu = gtk_menu_new ();
+
+	if (!commie_mode) {
+		GtkWidget *image;
+
+		menuitem = gtk_menu_item_new ();
+
+		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
+						  GTK_ICON_SIZE_MENU);
+
+		setup_menuitem (menuitem, image , _("Remove from panel"));
+
+		g_signal_connect (G_OBJECT (menuitem),
+				    "activate",
+				    G_CALLBACK (remove_applet_callback),
+				    info);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+
 		menuitem = gtk_menu_item_new();
-		gtk_menu_append(GTK_MENU(info->menu), menuitem);
+
+		/*
+		 * FIXME: should have a "Move" pixmap.
+		 */
+		setup_menuitem (menuitem, NULL, _("Move"));
+
+		g_signal_connect (G_OBJECT (menuitem),
+				    "activate",
+				    G_CALLBACK (move_applet_callback),
+				    info);
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+	}
+
+	menuitem = applet_setup_panel_menu (is_basep);
+
+	gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+
+	if (info->user_menu) {
+		menuitem = gtk_menu_item_new ();
+
+		gtk_menu_shell_append (GTK_MENU_SHELL (info->menu), menuitem);
+
 		gtk_widget_set_sensitive (menuitem, FALSE);
-		gtk_widget_show(menuitem);
+
+		gtk_widget_show (menuitem);
 	}
 
-	for(;user_menu!=NULL;user_menu = g_list_next(user_menu)) {
-		AppletUserMenu *menu=user_menu->data;
-		add_to_submenus (info, "", menu->name, menu, info->menu,
-				 info->user_menu);
+	for (l = info->user_menu; l; l = l->next) {
+		AppletUserMenu *menu = (AppletUserMenu *)l->data;
+
+		add_to_submenus (info, "", menu->name, menu, 
+				 info->menu, info->user_menu);
 	}
 
-	/*connect the deactivate signal, so that we can "re-allow" autohide
-	  when the menu is deactivated*/
-	gtk_signal_connect (GTK_OBJECT (info->menu), "deactivate",
-			    GTK_SIGNAL_FUNC (applet_menu_deactivate),
-			    info);
+	/*
+	 * connect the deactivate signal, so that we can "re-allow" 
+	 * autohide when the menu is deactivated.
+	 */
+	g_signal_connect (G_OBJECT (info->menu),
+			    "deactivate",
+			     G_CALLBACK (applet_menu_deactivate),
+			     info);
 }
 
 static void
-set_data(GtkWidget *menu, gpointer panel)
+applet_menu_set_data_recursively (GtkMenu     *menu,
+				  const gchar *key,
+				  gpointer     data)
 {
-	GList *children, *li;
+	GList *children;
+	GList *l;
 
-	gtk_object_set_data (GTK_OBJECT (menu), "menu_panel", panel);
+	gtk_object_set_data (GTK_OBJECT (menu), key, data);
 
 	children = gtk_container_children (GTK_CONTAINER (menu));
 
-	for(li = children; li; li = li->next) {
-		GtkMenuItem *item = li->data;
-		if (item->submenu != NULL)
-			set_data (item->submenu, panel);
+	for (l = children; l; l = l->next) {
+		GtkWidget *submenu = GTK_MENU_ITEM (l->data)->submenu;
+
+		if (submenu)
+			applet_menu_set_data_recursively (GTK_MENU (submenu), key, data);
 	}
+
 	g_list_free (children);
 }
 
-void
-show_applet_menu(AppletInfo *info, GdkEventButton *event)
+static void
+applet_show_menu (AppletInfo     *info,
+		  GdkEventButton *event)
 {
 	GtkWidget *panel;
 
-	g_return_if_fail(info!=NULL);
+	g_return_if_fail (info);
 
-	panel = get_panel_parent(info->widget);
+	panel = get_panel_parent (info->widget);
 
-	if (info->menu == NULL)
-		create_applet_menu (info, IS_BASEP_WIDGET (panel));
-	g_assert (info->menu != NULL);
+	if (!info->menu)
+		panel_applet_create_menu (info, BASEP_IS_WIDGET (panel));
 
-	if(IS_BASEP_WIDGET(panel)) {
-		BASEP_WIDGET(panel)->autohide_inhibit = TRUE;
-		basep_widget_queue_autohide(BASEP_WIDGET(panel));
+	g_assert (info->menu);
+
+	if (BASEP_IS_WIDGET (panel)) {
+		BASEP_WIDGET (panel)->autohide_inhibit = TRUE;
+		basep_widget_queue_autohide (BASEP_WIDGET (panel));
 	}
+
 	info->menu_age = 0;
 
-	set_data (info->menu, info->widget->parent);
+	applet_menu_set_data_recursively (GTK_MENU (info->menu),
+					  "menu_panel",
+					  info->widget->parent);
 
+	if (!GTK_WIDGET_REALIZED (info->menu))
+		gtk_widget_show (info->menu);
+
+	/* FIXME - off panel popups, should be automatic with new gtk-menu? */
 	gtk_menu_popup (GTK_MENU (info->menu),
 			NULL,
 			NULL,
-			global_config.off_panel_popups ?
-			  applet_menu_position :
-			  NULL,
+			applet_menu_position,
 			info,
 			event->button,
 			event->time);
 }
 
-
-
 static gboolean
-applet_button_press (GtkWidget *widget, GdkEventButton *event, AppletInfo *info)
+applet_button_press (GtkWidget      *widget,
+		     GdkEventButton *event,
+		     AppletInfo     *info)
 {
 	if (event->button == 3) {
-		if ( ! panel_applet_in_drag) {
-			if(info->type == APPLET_SWALLOW) {
-				Swallow *swallow = info->data;
-				GtkWidget *handle_box = swallow->handle_box;
-				if(!GTK_HANDLE_BOX(handle_box)->child_detached)
-					show_applet_menu(info, event);
-			} else {
-				show_applet_menu (info, event);
-			}
+		if (panel_applet_in_drag)
+			return FALSE;
+
+		if (info->type != APPLET_SWALLOW) 
+			applet_show_menu (info, event);
+
+		else {
+			GtkHandleBox *handle_box;
+
+			handle_box = GTK_HANDLE_BOX (((Swallow *)info->data)->handle_box);
+				
+			if (handle_box->child_detached)
+				applet_show_menu (info, event);
 		}
-	}
-	/* don't let any button click events to the panel or moving the applets
-	 * would move the panel */
-	return TRUE;
-}
 
-static GList *launchers_to_kill = NULL;
-
-void
-remove_unused_launchers (void)
-{
-	GList *li;
-
-	for (li = launchers_to_kill; li != NULL; li = li->next) {
-		char *file = li->data;
-		li->data = NULL;
-
-		unlink (file);
-
-		g_free (file);
+		return TRUE;
 	}
 
-	g_list_free (launchers_to_kill);
-	launchers_to_kill = NULL;
+	if (BUTTON_IS_WIDGET (widget))
+		return FALSE;
+	else
+		return TRUE;
 }
 
 static void
@@ -634,18 +675,12 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 			gtk_widget_destroy(dw);
 		}
 	} else if (info->type == APPLET_LAUNCHER) {
-		Launcher *launcher = info->data;
+		Launcher    *launcher = info->data;
+		const gchar *location;
 
-		/* we CAN'T unlink the file here as we may just
-		 * be killing stuff before exit, basically we
-		 * just want to schedule removals until session
-		 * saving */
+		location = gnome_desktop_item_get_location (launcher->ditem);
 
-		if (launcher->dentry->location != NULL) {
-			char *file = g_strdup (launcher->dentry->location);
-			launchers_to_kill = 
-				g_list_prepend (launchers_to_kill, file);
-		}
+		session_add_dead_launcher (location);
 	}
 
 
@@ -681,21 +716,20 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 	info->user_menu = NULL;
 }
 
-gboolean
-register_toy (GtkWidget *applet,
-	      gpointer data,
-	      GDestroyNotify data_destroy,
-	      PanelWidget *panel,
-	      int pos,
-	      gboolean exactpos,
-	      AppletType type)
+AppletInfo *
+panel_applet_register (GtkWidget      *applet,
+		       gpointer        data,
+		       GDestroyNotify  data_destroy,
+		       PanelWidget    *panel,
+		       gint            pos,
+		       gboolean        exactpos,
+		       AppletType      type)
 {
 	AppletInfo *info;
 	int newpos;
 	gboolean insert_at_pos;
 	
-	g_return_val_if_fail (applet != NULL, FALSE);
-	g_return_val_if_fail (panel != NULL, FALSE);
+	g_return_val_if_fail (applet && panel, NULL);
 
 	if ( ! GTK_WIDGET_NO_WINDOW (applet))
 		gtk_widget_set_events (applet, (gtk_widget_get_events (applet) |
@@ -728,12 +762,8 @@ register_toy (GtkWidget *applet,
 	gtk_object_set_data (GTK_OBJECT (applet),
 			     PANEL_APPLET_FORBIDDEN_PANELS, NULL);
 		
-	if (applets == NULL) {
-		applets_last = applets = g_slist_append (NULL, info);
-	} else {
-		applets_last = g_slist_append (applets_last, info);
-		applets_last = applets_last->next;
-	}
+	applets = g_slist_append (applets, info);
+
 	applet_count++;
 
 	/*we will need to save this applet's config now*/
@@ -762,28 +792,29 @@ register_toy (GtkWidget *applet,
 			/*can't put it anywhere, clean up*/
 			gtk_widget_destroy(applet);
 			info->widget = NULL;
-			panel_clean_applet(info);
+			panel_applet_clean(info);
 			g_warning(_("Can't find an empty spot"));
-			return FALSE;
+			g_free (info);
+			return NULL;
 		}
 		panel = PANEL_WIDGET(list->data);
 	}
 
-	if(IS_BUTTON_WIDGET (applet) || !GTK_WIDGET_NO_WINDOW(applet))
-		gtk_signal_connect(GTK_OBJECT(applet),
+	if(BUTTON_IS_WIDGET (applet) || !GTK_WIDGET_NO_WINDOW(applet))
+		g_signal_connect(G_OBJECT(applet),
 				   "button_press_event",
-				   GTK_SIGNAL_FUNC(applet_button_press),
+				   G_CALLBACK(applet_button_press),
 				   info);
 
-	gtk_signal_connect(GTK_OBJECT(applet), "destroy",
-			   GTK_SIGNAL_FUNC(applet_destroy),
+	g_signal_connect(G_OBJECT(applet), "destroy",
+			   G_CALLBACK(applet_destroy),
 			   info);
 
 	gtk_widget_show_all(applet);
 
-	/*gtk_signal_connect (GTK_OBJECT (applet), 
+	/*g_signal_connect (G_OBJECT (applet), 
 			    "drag_request_event",
-			    GTK_SIGNAL_FUNC(panel_dnd_drag_request),
+			    G_CALLBACK(panel_dnd_drag_request),
 			    info);
 
 	gtk_widget_dnd_drag_set (GTK_WIDGET(applet), TRUE,
@@ -793,5 +824,5 @@ register_toy (GtkWidget *applet,
 	size_change (info, panel);
 	back_change (info, panel);
 
-	return TRUE;
+	return info;
 }
