@@ -9,19 +9,22 @@
 
 #include "panel-widget.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
-#define TYPE_BASEP_WIDGET        (basep_widget_get_type ())
-#define BASEP_WIDGET(o)          (GTK_CHECK_CAST((o), TYPE_BASEP_WIDGET, BasePWidget))
-#define BASEP_WIDGET_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), TYPE_BASEP_WIDGET, BasePWidgetClass))
-#define IS_BASEP_WIDGET(o)       (GTK_CHECK_TYPE((o), TYPE_BASEP_WIDGET))
-#define IS_BASEP_WIDGET_CLASS(k) (GTK_CHECK_CLASS_TYPE((k), TYPE_BASEP_WIDGET))
+#define BASEP_TYPE_WIDGET        	(basep_widget_get_type ())
+#define BASEP_WIDGET(object)          	(G_TYPE_CHECK_INSTANCE_CAST ((object), BASEP_TYPE_WIDGET, BasePWidget))
+#define BASEP_WIDGET_CLASS(klass)    	(G_TYPE_CHECK_CLASS_CAST ((klass), BASEP_TYPE_WIDGET, BasePWidgetClass))
+#define BASEP_IS_WIDGET(object)       	(G_TYPE_CHECK_INSTANCE_TYPE ((object), BASEP_TYPE_WIDGET))
+#define BASEP_IS_WIDGET_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE ((klass), BASEP_TYPE_WIDGET))
+#define BASEP_WIDGET_GET_CLASS(object)	(G_TYPE_INSTANCE_GET_CLASS ((object), BASEP_TYPE_WIDGET, BasePWidgetClass))
+
+#define BASEP_TYPE_POS		        (basep_pos_get_type ())
+#define BASEP_POS(object)      		(G_TYPE_CHECK_INSTANCE_CAST((object), BASEP_TYPE_POS, BasePPos))
+#define BASEP_POS_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), BASEP_TYPE_POS, BasePPosClass))
+#define BASEP_IS_POS(object)            (G_TYPE_CHECK_INSTANCE_TYPE ((object), BASEP_TYPE_POS))
+#define BASEP_IS_POS_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), BASEP_TYPE_POS))
+#define BASEP_POS_GET_CLASS(k)		(BASEP_POS_CLASS(G_OBJECT_GET_CLASS(BASEP_WIDGET(k)->pos)))
 
-#define TYPE_BASEP_POS        (basep_pos_get_type ())
-#define BASEP_POS(o)          (GTK_CHECK_CAST((o), TYPE_BASEP_POS, BasePPos))
-#define BASEP_POS_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), TYPE_BASEP_POS, BasePPosClass))
-#define IS_BASEP_POS(o)       (GTK_CHECK_TYPE((o), TYPE_BASEP_POS))
-#define IS_BASEP_POS_CLASS(k) (GTK_CHECK_CLASS_TYPE((k), TYPE_BASEP_POS))
 
 typedef struct _BasePWidget		BasePWidget;
 typedef struct _BasePWidgetClass	BasePWidgetClass;
@@ -132,7 +135,7 @@ struct _BasePPosClass {
 
 	void (*update_winhints) (BasePWidget *basep);
 
-	PanelOrientType
+	PanelOrient
 	(*get_applet_orient) (BasePWidget *basep);
 
 	void (*set_pos)                   (BasePWidget *basep,
@@ -147,14 +150,14 @@ struct _BasePPosClass {
 	void (*get_size)                  (BasePWidget *basep,
 					   int *w, int *h);
 
-	PanelOrientType (*get_hide_orient) (BasePWidget *basep);
+	PanelOrient (*get_hide_orient) (BasePWidget *basep);
 
 	void (*get_hide_size)              (BasePWidget *basep,
-					    PanelOrientType hide_orient,
+					    PanelOrient hide_orient,
 					    int *w, int *h);
 
 	void (*get_hide_pos)               (BasePWidget *basep,
-					    PanelOrientType hide_orient,
+					    PanelOrient hide_orient,
 					    int *x, int *y,
 					    int w, int h);
 
@@ -175,8 +178,9 @@ struct _BasePPosClass {
 	void (*pre_convert_hook)          (BasePWidget *basep);
 };
 
-GtkType         basep_pos_get_type              (void) G_GNUC_CONST;
-GtkType		basep_widget_get_type		(void) G_GNUC_CONST;
+GType           basep_pos_get_type              (void) G_GNUC_CONST;
+GType		basep_widget_get_type		(void) G_GNUC_CONST;
+
 GtkWidget*	basep_widget_construct		(BasePWidget *basep,
 						 gboolean packed,
 						 gboolean reverse_arrows,
@@ -243,17 +247,17 @@ void            basep_widget_explicit_hide      (BasePWidget *basep,
 void            basep_widget_explicit_show      (BasePWidget *basep);
 
 void		basep_widget_do_hiding		(BasePWidget *basep,
-						 PanelOrientType hide_orient,
+						 PanelOrient hide_orient,
 						 int leftover,
 						 int step);
 
 void		basep_widget_do_showing		(BasePWidget *basep,
-						 PanelOrientType hide_orient,
+						 PanelOrient hide_orient,
 						 int leftover,
 						 int step);
 
 void		basep_widget_get_position	(BasePWidget *basep,
-						 PanelOrientType hide_orient,
+						 PanelOrient hide_orient,
 						 int *x, int *y,
 						 int w, int h);
 
@@ -263,7 +267,7 @@ void            basep_widget_get_menu_pos  (BasePWidget *basep,
 					    int wx, int wy,
 					    int ww, int wh);
 
-PanelOrientType basep_widget_get_applet_orient (BasePWidget *basep);
+PanelOrient basep_widget_get_applet_orient (BasePWidget *basep);
 
 /* initialize drag offsets according to cursor */
 void            basep_widget_init_offsets      (BasePWidget *basep);
@@ -277,15 +281,15 @@ void            basep_widget_get_pos           (BasePWidget *basep,
 void            basep_widget_get_size          (BasePWidget *basep,
 						int *w, int *h);
 
-PanelOrientType basep_widget_get_hide_orient   (BasePWidget *basep);
+PanelOrient basep_widget_get_hide_orient   (BasePWidget *basep);
 
 /*
 void            basep_widget_get_hide_size      (BasePWidget *basep,
-						 PanelOrientType hide_orient,
+						 PanelOrient hide_orient,
 						 int *w, int *h);
 
 void            basep_widget_get_hide_pos      (BasePWidget *basep,
-						PanelOrientType hide_orient,
+						PanelOrient hide_orient,
 						int *w, int *h);
 						*/
 
@@ -312,8 +316,10 @@ void		basep_border_get		(int screen,
 						 int *center,
 						 int *right);
 
+/* FIXME:
 #define GNOME_PANEL_HINTS (WIN_HINTS_SKIP_FOCUS|WIN_HINTS_SKIP_WINLIST|WIN_HINTS_SKIP_TASKBAR)
+*/
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif /* __BASEP_WIDGET_H__ */
