@@ -4,7 +4,7 @@
 #include "panel.h"
 #include "panel_config.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 #define DRAWER_PANEL_KEY "drawer_panel"
 
@@ -16,14 +16,17 @@ typedef struct {
 	GtkWidget *properties;
 } Drawer;
 
-gboolean load_drawer_applet (int mypanel, const char *pixmap,
-			     const char *tooltip,
-			     PanelWidget *panel, int pos, gboolean exactpos);
+Drawer *load_drawer_applet (int          mypanel,
+			    const char  *pixmap,
+			    const char  *tooltip,
+			    PanelWidget *panel,
+			    int          pos,
+			    gboolean     exactpos);
 
-void set_drawer_applet_orient(Drawer *drawer, PanelOrientType orient);
+void set_drawer_applet_orient(Drawer *drawer, PanelOrient orient);
 
 void add_drawer_properties_page(PerPanelConfig *ppc, GtkNotebook *prop_nbook, Drawer *drawer);
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
