@@ -3,12 +3,19 @@
 /* uncorbized by George Lebl */
 
 #include <config.h>
-#include <gnome.h>
+#include <string.h>
 
-#include "panel-include.h"
+#include <libgnome/libgnome.h>
 
-extern GSList *applets;
-extern GSList *applets_last;
+#include "logout.h"
+
+#include "applet.h"
+#include "drawer-widget.h"
+#include "menu.h"
+#include "panel_config_global.h"
+#include "panel.h"
+#include "extern.h"
+#include "session.h"
 
 extern GtkTooltips *panel_tooltips;
 
@@ -18,11 +25,10 @@ extern gboolean commie_mode;
 static void
 logout (GtkWidget *widget)
 {
-	gtk_signal_handler_block_by_func (GTK_OBJECT (widget), GTK_SIGNAL_FUNC(logout), NULL);
 	if (global_config.drawer_auto_close) {
 		GtkWidget *parent = PANEL_WIDGET(widget->parent)->panel_parent;
 		g_return_if_fail(parent!=NULL);
-		if(IS_DRAWER_WIDGET(parent)) {
+		if(DRAWER_IS_WIDGET(parent)) {
 			BasePWidget *basep = BASEP_WIDGET(parent);
 			GtkWidget *grandparent = PANEL_WIDGET(basep->panel)->master_widget->parent;
 			GtkWidget *grandparentw =
@@ -33,7 +39,6 @@ logout (GtkWidget *widget)
 	}
 
 	panel_quit();
-	gtk_signal_handler_unblock_by_func (GTK_OBJECT (widget), GTK_SIGNAL_FUNC(logout), NULL);
 }
 
 static void  
@@ -67,12 +72,13 @@ create_logout_widget (void)
 	GtkWidget *button;
 	char *pixmap_name;
 
-	pixmap_name = gnome_pixmap_file ("gnome-term-night.png");
+	pixmap_name = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+						 "gnome-term-night.png", TRUE, NULL);
 
 	button = button_widget_new (pixmap_name, -1,
-				    MISC_TILE,
+				    MISC_POBJECT,
 				    FALSE,
-				    ORIENT_UP,
+				    PANEL_ORIENT_UP,
 				    _("Log out"));
 
 	/*A hack since this function only pretends to work on window
@@ -87,35 +93,37 @@ create_logout_widget (void)
 			     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 
-	gtk_signal_connect (GTK_OBJECT (button), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get_cb),
+	g_signal_connect (G_OBJECT (button), "drag_data_get",
+			    G_CALLBACK (drag_data_get_cb),
 			    "LOGOUT");
 
 	g_free (pixmap_name);
 	gtk_tooltips_set_tip (panel_tooltips, button, _("Log out of GNOME"), NULL);
 
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (logout), NULL);
+	g_signal_connect (G_OBJECT (button), "clicked",
+			    G_CALLBACK (logout), NULL);
 
 	return button;
 }
 
 void
-load_logout_applet(PanelWidget *panel, int pos, gboolean exactpos)
+load_logout_applet (PanelWidget *panel,
+		    gint         pos,
+		    gboolean     exactpos)
 {
-	GtkWidget *logout;
+	GtkWidget  *logout;
+	AppletInfo *info;
 
-	logout = create_logout_widget();
-	if(!logout)
+	logout = create_logout_widget ();
+	if (!logout)
 		return;
 
-	if (!register_toy(logout, NULL, NULL, panel,
-			  pos, exactpos, APPLET_LOGOUT))
+	info = panel_applet_register (logout, NULL, NULL, panel,
+				      pos, exactpos, APPLET_LOGOUT);
+	if (!info)
 		return;
 
-	applet_add_callback(applets_last->data, "help",
-			    GNOME_STOCK_PIXMAP_HELP,
-			    _("Help"));
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("Help"));
 }
 
 static GtkWidget *
@@ -127,12 +135,14 @@ create_lock_widget(void)
 	GtkWidget *button;
 	char *pixmap_name;
 
-	pixmap_name = gnome_pixmap_file("gnome-lockscreen.png");
+	pixmap_name = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+						 "gnome-lockscreen.png",
+						 TRUE, NULL);
 
 	button = button_widget_new (pixmap_name, -1,
-				    MISC_TILE,
+				    MISC_POBJECT,
 				    FALSE,
-				    ORIENT_UP,
+				    PANEL_ORIENT_UP,
 				    _("Lock screen"));
 
 	/*A hack since this function only pretends to work on window
@@ -147,29 +157,35 @@ create_lock_widget(void)
 			     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 	GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 
-	gtk_signal_connect (GTK_OBJECT (button), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get_cb),
+	g_signal_connect (G_OBJECT (button), "drag_data_get",
+			    G_CALLBACK (drag_data_get_cb),
 			    "LOCK");
 
 	g_free (pixmap_name);
 	gtk_tooltips_set_tip (panel_tooltips, button, _("Lock screen"), NULL);
 
-	gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			    GTK_SIGNAL_FUNC (panel_lock), NULL);
+	g_signal_connect (G_OBJECT (button), "clicked",
+			    G_CALLBACK (panel_lock), NULL);
 
 	return button;
 }
 
 void
-load_lock_applet(PanelWidget *panel, int pos, gboolean exactpos)
+load_lock_applet (PanelWidget *panel,
+		  gint         pos,
+		  gboolean     exactpos)
 {
-	GtkWidget *lock;
-	lock = create_lock_widget();
+	GtkWidget  *lock;
+	AppletInfo *info;
 
-	if(!lock)
+	lock = create_lock_widget ();
+
+	if (!lock)
 		return;
-	if (!register_toy(lock, NULL, NULL, panel, pos, 
-			  exactpos, APPLET_LOCK))
+
+	info = panel_applet_register (lock, NULL, NULL, panel, pos,
+				      exactpos, APPLET_LOCK);
+	if (!info)
 		return;
 
         /*
@@ -184,20 +200,13 @@ load_lock_applet(PanelWidget *panel, int pos, gboolean exactpos)
 	  <jwz> and "xscreensaver-demo"
 	*/
 
-	applet_add_callback(applets_last->data, "activate",
-			    NULL, _("Blank Screen Now"));
-	applet_add_callback(applets_last->data, "lock",
-			    NULL, _("Lock Screen Now"));
-	applet_add_callback(applets_last->data, "exit",
-			    NULL, _("Kill Daemon"));
-	applet_add_callback(applets_last->data, "restart",
-			    NULL, _("Restart Daemon"));
-	if ( ! commie_mode)
-		applet_add_callback(applets_last->data, "prefs",
-				    NULL, _("Preferences"));
-	applet_add_callback(applets_last->data, "help",
-			    GNOME_STOCK_PIXMAP_HELP,
-			    _("Help"));
+	panel_applet_add_callback (info, "activate", NULL, _("Blank Screen Now"));
+	panel_applet_add_callback (info, "lock", NULL, _("Lock Screen Now"));
+	panel_applet_add_callback (info, "exit", NULL, _("Kill Daemon"));
+	panel_applet_add_callback (info, "restart", NULL, _("Restart Daemon"));
+
+	if (!commie_mode)
+		panel_applet_add_callback (info, "prefs", NULL, _("Preferences"));
+
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("Help"));
 }
-
-
