@@ -11,36 +11,41 @@
 #include <string.h>
 #include <signal.h>
 #include <sys/wait.h>
-#include <gnome.h>
-#include <libgnorba/gnorba.h>
-#include <libgnomeui/gnome-window-icon.h>
 
-#include "panel-include.h"
+#include <libgnome/libgnome.h>
+#include <libgnomeui/libgnomeui.h>
 
-#include "gwmh.h"
+#include <libbonoboui.h>
+#include <bonobo-activation/bonobo-activation.h>
+
+#include "main.h"
 
-#include "xstuff.h"
-#include "multiscreen-stuff.h"
 #include "conditional.h"
+#include "drawer-widget.h"
+#include "extern.h"
+#include "menu-fentry.h"
+#include "menu.h"
+#include "multiscreen-stuff.h"
+#include "panel-util.h"
+#include "panel-gconf.h"
+#include "panel_config_global.h"
+#include "session.h"
+#include "status.h"
+#include "xstuff.h"
+
+/*#include "panel-unique-factory.h"*/
+#include "panel-shell.h"
 
 extern int config_sync_timeout;
-extern int applets_to_sync;
-extern int panels_to_sync;
-extern int need_complete_save;
 
 extern GSList *panels;
 
 extern GSList *applets;
-extern GSList *applets_last;
-extern int applet_count;
 
 extern gboolean commie_mode;
 extern GlobalConfig global_config;
-extern char *panel_cfg_path;
 extern char *old_panel_cfg_path;
 
-extern GdkPixmap *desktop_pixmap;
-
 /*list of all panel widgets created*/
 extern GSList *panel_list;
 
@@ -52,10 +57,6 @@ char *kde_menudir = NULL;
 char *kde_icondir = NULL;
 char *kde_mini_icondir = NULL;
 
-char *merge_merge_dir = NULL;
-int merge_main_dir_len = 0;
-char *merge_main_dir = NULL;
-
 gboolean panel_in_startup = TRUE;
 
 static gboolean
@@ -71,7 +72,7 @@ menu_age_timeout(gpointer data)
 			info->menu_age = 0;
 		}
 		/*if we are allowed to, don't destroy applet menus*/
-		if(!global_config.hungry_menus &&
+		if(!global_config.keep_menus_in_memory &&
 		   info->type == APPLET_MENU) {
 			Menu *menu = info->data;
 			if(menu->menu && menu->age++>=6 &&
@@ -84,7 +85,7 @@ menu_age_timeout(gpointer data)
 	}
 	
 	/*skip panel menus if we are memory hungry*/
-	if(global_config.hungry_menus)
+	if(global_config.keep_menus_in_memory)
 		return TRUE;
 	
 	for(li = panel_list; li != NULL; li = g_slist_next(li)) {
@@ -234,12 +235,16 @@ check_screen (void)
 
 	name = g_strdup_printf ("%cish/%cishanim.png",
 				'f', 'f');
-	phsh_file = gnome_pixmap_file (name);
+
+	phsh_file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+					       name, TRUE, NULL);
+
 	g_free (name);
-	if (phsh_file == NULL)
+
+	if (!phsh_file)
 		return;
 
-	tmp = gdk_pixbuf_new_from_file (phsh_file);
+	tmp = gdk_pixbuf_new_from_file (phsh_file, NULL);
 	if (tmp == NULL)
 		return;
 
@@ -335,11 +340,13 @@ try_config_sync(gpointer data)
 	return TRUE;
 }
 
+/* Note: similar function is in gnome-desktop-item !!! */
+
 static void
-find_kde_directory(void)
+find_kde_directory (void)
 {
 	int i;
-	char *kdedir = g_getenv ("KDEDIR");
+	const char *kdedir = g_getenv ("KDEDIR");
 	char *try_prefixes[] = {
 		"/usr",
 		"/opt/kde",
@@ -347,28 +354,28 @@ find_kde_directory(void)
 		"/kde",
 		NULL
 	};
-	if(kdedir) {
-		kde_menudir = g_concat_dir_and_file(kdedir,"share/applnk");
-		kde_icondir = g_concat_dir_and_file(kdedir,"share/icons");
-		kde_mini_icondir = g_concat_dir_and_file(kdedir,"share/icons/mini");
+	if (kdedir != NULL) {
+		kde_menudir = g_build_filename (kdedir, "share", "applnk", NULL);
+		kde_icondir = g_build_filename (kdedir, "share", "icons", NULL);
+		kde_mini_icondir = g_build_filename (kdedir, "share", "icons", "mini", NULL);
 		return;
 	}
 
 	/* if what configure gave us works use that */
-	if(g_file_test(KDE_MENUDIR,G_FILE_TEST_ISDIR)) {
-		kde_menudir = g_strdup(KDE_MENUDIR);
-		kde_icondir = g_strdup(KDE_ICONDIR);
-		kde_mini_icondir = g_strdup(KDE_MINI_ICONDIR);
+	if (g_file_test (KDE_MENUDIR, G_FILE_TEST_IS_DIR)) {
+		kde_menudir = g_strdup (KDE_MENUDIR);
+		kde_icondir = g_strdup (KDE_ICONDIR);
+		kde_mini_icondir = g_strdup (KDE_MINI_ICONDIR);
 		return;
 	}
 
-	for(i=0;try_prefixes[i];i++) {
+	for (i = 0; try_prefixes[i] != NULL; i++) {
 		char *try;
-		try = g_concat_dir_and_file(try_prefixes[i],"share/applnk");
-		if(g_file_test(try,G_FILE_TEST_ISDIR)) {
+		try = g_build_filename (try_prefixes[i], "share", "applnk", NULL);
+		if (g_file_test (try, G_FILE_TEST_IS_DIR)) {
 			kde_menudir = try;
-			kde_icondir = g_concat_dir_and_file(try_prefixes[i],"share/icons");
-			kde_mini_icondir = g_concat_dir_and_file(try_prefixes[i],"share/icons/mini");
+			kde_icondir = g_build_filename (try_prefixes[i], "share", "icons", NULL);
+			kde_mini_icondir = g_build_filename (try_prefixes[i], "share", "icons", "mini", NULL);
 			return;
 		}
 		g_free(try);
@@ -376,199 +383,157 @@ find_kde_directory(void)
 
 	/* absolute fallback, these don't exist, but we're out of options
 	   here */
-	kde_menudir = g_strdup(KDE_MENUDIR);
-	kde_icondir = g_strdup(KDE_ICONDIR);
-	kde_mini_icondir = g_strdup(KDE_MINI_ICONDIR);
-}
-
-static void
-setup_merge_directory(void)
-{
-	int len;
-
-	merge_main_dir = gnome_datadir_file("gnome/apps");
-	merge_main_dir_len = merge_main_dir != NULL ? strlen (merge_main_dir) : 0;
-	merge_merge_dir = conditional_get_string ("/panel/Merge/Directory",
-						  "/etc/X11/applnk/",
-						  NULL);
-
-	if (string_empty (merge_merge_dir) ||
-	    ! g_file_test(merge_merge_dir, G_FILE_TEST_ISDIR)) {
-		g_free(merge_merge_dir);
-		merge_merge_dir = NULL;
-		return;
-	}
-
-	len = strlen(merge_merge_dir);
-	if (merge_merge_dir[len-1] != '/') {
-		char *tmp = g_strconcat(merge_merge_dir, "/", NULL);
-		g_free(merge_merge_dir);
-		merge_merge_dir = tmp;
-	}
+	kde_menudir = g_strdup (KDE_MENUDIR);
+	kde_icondir = g_strdup (KDE_ICONDIR);
+	kde_mini_icondir = g_strdup (KDE_MINI_ICONDIR);
 }
 
 static void
 setup_visuals (void)
 {
 	gdk_rgb_init ();
-	gtk_widget_push_visual (gdk_rgb_get_visual ());
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 }
 
 static void
 kill_free_drawers (void)
 {
-	GSList *li, *list;
-	list = g_slist_copy (panel_list); 
-	for (li = list; li != NULL; li = li->next) {
+	GSList *li;
+	GSList *to_destroy = NULL;
+	
+	for (li = panel_list; li != NULL; li = li->next) {
 		PanelData *pd = li->data;
-		if (IS_DRAWER_WIDGET (pd->panel) &&
+		if (DRAWER_IS_WIDGET (pd->panel) &&
 		    PANEL_WIDGET (BASEP_WIDGET (pd->panel)->panel)->master_widget == NULL) {
 			status_unparent (pd->panel);
-			gtk_widget_destroy (pd->panel);
+			to_destroy = g_slist_prepend (to_destroy, pd->panel);
 		}
 	}
-	g_slist_free(list);
+
+	g_slist_foreach (to_destroy, (GFunc)gtk_widget_destroy, NULL);
+	g_slist_free (to_destroy);
 }
 
-/* gets a GdkPixmap for a given X atom name whose value is an X Pixmap */
-static GdkPixmap *
-get_pixmap_prop (char *prop_id)
-{
-	Atom prop, type;
-	int format;
-	unsigned long length, after;
-	unsigned char *data;
-	Pixmap p;
-
-	prop = XInternAtom(GDK_DISPLAY(), prop_id, True);
-
-	if (prop == None) {
-		return NULL;
-	}
-
-	data = NULL;
-	XGetWindowProperty(GDK_DISPLAY(), GDK_ROOT_WINDOW(), prop, 0L, 1L, 
-			False, AnyPropertyType, &type, &format, &length, 
-			&after, &data);
-
-	if (type != XA_PIXMAP) {
-		if (data)
-			XFree(data);
-		return NULL;
-	}
-
-	p = *((Pixmap *)data);
-
-	if (data)
-    		XFree(data);
-
-	/* remember not to unref this pixmap */
-	return gdk_pixmap_foreign_new(p);
-}
-
-
-/* gets a GdkPixmap that refers to the GNOME desktop pixmap and stores it
- * in desktop_pixmap.
- */
 static void
-get_desktop_pixmap()
+tell_user_Im_on_crack (void)
 {
-	desktop_pixmap = get_pixmap_prop ("_XROOTPMAP_ID");
+	GtkWidget *dialog;
+
+	dialog = gtk_message_dialog_new
+	    (NULL /* parent */,
+	     0 /* flags */,
+	     GTK_MESSAGE_WARNING,
+	     GTK_BUTTONS_YES_NO,
+	     _("This is the GNOME2 panel, it is likely that it will crash,\n"
+	       "destroy configuration, cause another world war, and most\n"
+	       "likely just plain not work.  Use at your own risk.\n\n"
+	       "Do you really want to run it?"));
+
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES)
+		exit (0);
+
+	gtk_widget_destroy (dialog);
 }
 
-/* an event handler for when the GNOME desktop pixmap changes */
-static GdkFilterReturn
-desktop_event_filter (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
+static PanelShell *
+panel_get_shell (void)
 {
-	XEvent *xevent;
-	GSList *item;
-	PanelWidget *panel;
+	static PanelShell *shell = NULL;
 
-	xevent = (XEvent *) gdk_xevent;
+	if (!shell) {
+		shell = g_object_new (PANEL_SHELL_TYPE, NULL);
+		bonobo_object_set_immortal (BONOBO_OBJECT (shell), TRUE);
+	}	
 
-	if (xevent->type == PropertyNotify && xevent->xproperty.atom == 
-			gdk_atom_intern("ESETROOT_PMAP_ID", TRUE)) {
-
-		get_desktop_pixmap ();
-
-		for (item = panels; item; item = g_slist_next (item)) {
-			panel = PANEL_WIDGET (item->data);
-
-			if (panel->back_type == PANEL_BACK_TRANSLUCENT) {
-				panel_widget_setup_translucent_background (panel);
-				panel_widget_force_repaint (panel);
-			}
-		}
-
-
-	}
-	return GDK_FILTER_CONTINUE;
+	return shell;
 }
 
-/* set up the event handler */
-static void
-set_up_desktop_event_handler () 
-{
-	/* we need to watch for desktop image changes */
-	gdk_window_add_filter (GDK_ROOT_PARENT(), desktop_event_filter, NULL);
-}	
-
-
 int
 main(int argc, char **argv)
 {
-	CORBA_ORB orb;
-	CORBA_Environment ev;
-	gboolean duplicate;
 	gchar *real_global_path;
+	PanelShell *shell;
+	Bonobo_RegistrationResult reg_res;
+	char *message = NULL;
 	
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
+	gnome_program_init ("panel", VERSION,
+			    LIBGNOMEUI_MODULE,
+			    argc, argv,
+			    GNOME_PROGRAM_STANDARD_PROPERTIES,
+			    NULL);
+
+	bonobo_activate ();
+
+	tell_user_Im_on_crack ();
+
+	/*
+	 * Let applets spew.
+	 */
+	putenv ("BONOBO_ACTIVATION_DEBUG_OUTPUT=1");
+
+	switch (extern_init ()) {
+	case EXTERN_SUCCESS: 
+		break;
+	case EXTERN_ALREADY_ACTIVE:
+		message = _("I've detected a panel already running,\n"
+			    "and will now exit.");
+		break;
+	case EXTERN_FAILURE: {
+		message = _("There was a problem registering the panel "
+			    "with the bonobo-activation server.\n"
+			    "The panel will now exit.");
+		break;
+		}
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	shell = panel_get_shell ();
+
+	reg_res = bonobo_activation_active_server_register (
+		"OAFIID:GNOME_PanelShell",
+		BONOBO_OBJREF (shell));
+	
+	switch (reg_res) {
+	case Bonobo_ACTIVATION_REG_SUCCESS:
+		/* whee! success! */
+		break;
+	case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE:
+		message = _("I've detected a panel already running,\n"
+			    "and will now exit.");
+		break;
+	default:
+		message = _("There was a problem registering the panel "
+			    "with the bonobo-activation server.\n"
+			    "The panel will now exit.");
+		break;
+	}
+
+	if (message) {
+		GtkWidget *box;
+
+		box = gtk_message_dialog_new (NULL,
+					      GTK_DIALOG_MODAL,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_OK,
+					      message);
+		gtk_dialog_run (GTK_DIALOG (box));
+		gtk_widget_destroy (box);
+
+		return -1;
+	}
 
-	CORBA_exception_init(&ev);
-	orb = gnome_CORBA_init("panel", VERSION,
-			       &argc, argv,
-			       GNORBA_INIT_SERVER_FUNC, &ev);
-	CORBA_exception_free(&ev);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-panel.png");
 	setup_visuals ();
 
-	switch (panel_corba_gtk_init (orb)) {
-	case 0: 
-		duplicate = FALSE;
-		break; /* success */
-	case -4: {
-		GtkWidget* box = gnome_question_dialog
-			(_("I've detected a panel already running.\n"
-			   "Start another panel as well?\n" 
-			   "(The new panel will not be restarted.)"), NULL, NULL);
-		panel_set_dialog_layer (box);
-		if (gnome_dialog_run_and_close (GNOME_DIALOG (box)))
-			return 0;
-		duplicate = TRUE;
-		break;
-	}
-	default: {
-		GtkWidget *box = panel_error_dialog
-			(_("There was a problem registering the panel "
-			   "with the GOAD server.\n"
-			   "The panel will now exit."));
-		gnome_dialog_run_and_close (GNOME_DIALOG (box));
-		return 0;
-		break;
-	}
-	}
-
-	setup_merge_directory();
-
 	find_kde_directory();
 
 	client = gnome_master_client ();
 
-	gnome_client_set_restart_style (client, duplicate 
-					? GNOME_RESTART_NEVER 
-					: GNOME_RESTART_IMMEDIATELY);
+	gnome_client_set_restart_style (client, GNOME_RESTART_IMMEDIATELY);
 
 	gnome_client_set_priority (client, 40);
 
@@ -576,39 +541,35 @@ main(int argc, char **argv)
 	if (gnome_client_get_flags (client) & GNOME_CLIENT_RESTORED)
 		old_panel_cfg_path = g_strdup (gnome_client_get_config_prefix (client));
 	else
-		old_panel_cfg_path = g_strdup ("/panel.d/default/");
+		old_panel_cfg_path = g_strdup (PANEL_CONFIG_PATH);
 
 #ifndef PER_SESSION_CONFIGURATION
 	real_global_path = gnome_config_get_real_path (old_panel_cfg_path);
-	if ( ! panel_file_exists (real_global_path)) {
+	if ( ! g_file_test (real_global_path, G_FILE_TEST_EXISTS)) {
 		g_free (old_panel_cfg_path);
-		old_panel_cfg_path = g_strdup ("/panel.d/default/");
+		old_panel_cfg_path = g_strdup (PANEL_CONFIG_PATH);
 	}
 	g_free (real_global_path);
 #endif /* !PER_SESSION_CONFIGURATION */
 
 	gnome_client_set_global_config_prefix (client, PANEL_CONFIG_PATH);
 	
-	gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
-			    GTK_SIGNAL_FUNC (panel_session_save), NULL);
-	gtk_signal_connect (GTK_OBJECT (client), "die",
-			    GTK_SIGNAL_FUNC (panel_session_die), NULL);
+	g_signal_connect (G_OBJECT (client), "save_yourself",
+			  G_CALLBACK (panel_session_save), NULL);
+	g_signal_connect (G_OBJECT (client), "die",
+			  G_CALLBACK (panel_session_die), NULL);
 
 	panel_tooltips = gtk_tooltips_new ();
 
 	xstuff_init ();
-	status_applet_create_offscreen ();
-
 	multiscreen_init ();
 
+#if FIXME
 	gnome_win_hints_init ();
+#endif
 
 	load_system_wide ();
 
-	/* read, convert and remove old config */
-	if ( ! commie_mode)
-		convert_old_config ();
-
 	/* set the globals, it is important this is before
 	 * init_user_applets */
 	load_up_globals ();
@@ -616,23 +577,18 @@ main(int argc, char **argv)
 	if ( ! commie_mode)
 		write_global_config ();
 
-	init_fr_chunks ();
+#ifdef FIXME
+	gwmh_init ();
+#endif
 	
 	init_menus ();
 
-	/* get the GNOME desktop image stuff */
-	get_desktop_pixmap ();
-	set_up_desktop_event_handler ();
-	
-	init_user_panels ();
-	init_user_applets ();
+	load_session ();	
 
 	kill_free_drawers ();
 
 	load_tornoff ();
 
-	gnome_triggers_do ("Session startup", NULL, "gnome", "login", NULL);
-
 	/*add forbidden lists to ALL panels*/
 	g_slist_foreach (panels,
 			 (GFunc)panel_widget_add_forbidden,
@@ -652,7 +608,9 @@ main(int argc, char **argv)
 	
 	/*load these as the last thing to prevent some races any races from
 	  starting multiple goad_id's at once are libgnorba's problem*/
-	load_queued_externs ();
+	extern_load_queued ();
+
+	status_applet_create_offscreen ();
 
 	gtk_main ();
 
