@@ -2,10 +2,11 @@
 #define PANEL_H
 
 #include <gtk/gtk.h>
+#include "basep-widget.h"
 #include "panel-widget.h"
 #include "applet.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
 typedef struct _PanelData PanelData;
 struct _PanelData {
@@ -22,7 +23,7 @@ void orientation_change(AppletInfo *info, PanelWidget *panel);
 void size_change(AppletInfo *info, PanelWidget *panel);
 void back_change(AppletInfo *info, PanelWidget *panel);
 
-PanelOrientType get_applet_orient(PanelWidget *panel);
+PanelOrient get_applet_orient(PanelWidget *panel);
 
 void panel_setup(GtkWidget *panel);
 void basep_pos_connect_signals (BasePWidget *basep);
@@ -39,11 +40,14 @@ GtkWidget * make_popup_panel_menu (PanelWidget *panel);
 PanelData * panel_data_by_id (int id);
 void panel_set_id (GtkWidget *widget, int id);
 
+void status_unparent (GtkWidget *widget);
+
+
 
 #define get_panel_parent(appletw) \
 	 (PANEL_WIDGET(GTK_WIDGET(appletw)->parent)->panel_parent)
 
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
