@@ -2,17 +2,16 @@
 #define SESSION_H
 
 #include <glib.h>
+#include <libgnomeui/gnome-client.h>
 #include "panel-widget.h"
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
 
-#define DEFAULT_AUTO_HIDE_STEP_SIZE 10
-#define DEFAULT_EXPLICIT_HIDE_STEP_SIZE 80
-#define DEFAULT_DRAWER_STEP_SIZE 80
+#define DEFAULT_HIDE_SPEED 50
 
 /* amount of time in ms. to wait before lowering panel */
-#define DEFAULT_MINIMIZE_DELAY 300
-#define DEFAULT_MAXIMIZE_DELAY 0
+#define DEFAULT_HIDE_DELAY 300
+#define DEFAULT_SHOW_DELAY 0
 
 /* number of pixels it'll stick up from the bottom when using
    PANEL_AUTO_HIDE */
@@ -20,7 +19,7 @@ BEGIN_GNOME_DECLS
 
 #define DEFAULT_PANEL_NUM 0
 
-#define PANEL_CONFIG_PATH "/panel.d/default/"
+#define PANEL_CONFIG_PATH "panel2.d/default/"
 
 int panel_session_save (GnomeClient *client,
 			int phase,
@@ -46,8 +45,7 @@ void load_up_globals (void);
 /* only to be used on beginning, we don't really want to write globals much */
 void write_global_config (void);
 
-void init_user_applets(void);
-void init_user_panels(void);
+void load_session (void);
 
 void apply_global_config(void);
 
@@ -58,6 +56,8 @@ void convert_old_config(void);
 char * get_correct_prefix (char const **sep);
 void push_correct_global_prefix (void);
 
-END_GNOME_DECLS
+void session_add_dead_launcher (const gchar *location);
+
+G_END_DECLS
 
 #endif
