@@ -12,22 +12,35 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
-#include <gnome.h>
+
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 
-#include "panel-include.h"
-#include "panel-widget.h"
+#include <libgnomeui/libgnomeui.h>
+#include <libgnome/libgnome.h>
 
+#include "gnome-desktop-item.h"
+
+#include "swallow.h"
+
+#include "applet.h"
+#include "extern.h"
+#include "panel.h"
+#include "panel-util.h"
+#include "panel-widget.h"
 #include "xstuff.h"
 
 /* from gtkhandlebox.c */
 #define DRAG_HANDLE_SIZE 10
 
-GList *check_swallows = NULL;
+enum {
+	OK_BUTTON,
+	HELP_BUTTON,
+	RELOAD_BUTTON,
+	CANCEL_BUTTON
+};
 
-extern GlobalConfig global_config;
-extern GSList *applets_last;
+GList *check_swallows = NULL;
 
 static void socket_destroyed (GtkWidget *w, gpointer data);
 static void socket_realized (GtkWidget *w, gpointer data);
@@ -52,21 +65,28 @@ socket_realized (GtkWidget *w, gpointer data)
 static void
 swallow_launch (Swallow *swallow)
 {
-	if ( ! string_empty (swallow->path)) {
+	if (swallow->path && swallow->path [0] != '\0') {
 		char *p = strrchr (swallow->path, '.');
-		GnomeDesktopEntry *item;
+		GnomeDesktopItem *item;
 
 		/*only if such a file exists and ends in a .desktop, should
 		  we try to launch it as such*/
 		if(p != NULL &&
 		   (strcmp (p, ".desktop") == 0 ||
 		    strcmp (p, ".kdelnk") == 0) &&
-		   panel_file_exists (swallow->path) &&
-		   (item = gnome_desktop_entry_load (swallow->path)) != NULL) {
-			gnome_desktop_entry_launch (item);
-			gnome_desktop_entry_free (item);
+		   g_file_test (swallow->path, G_FILE_TEST_EXISTS) &&
+		   (item = gnome_desktop_item_new_from_file (swallow->path, 0, NULL)) != NULL) {
+			char *curdir = g_get_current_dir ();
+			chdir (g_get_home_dir ());
+
+			gnome_desktop_item_launch (item, 0, NULL, NULL);
+			/* FIXME: handle_errors */
+			gnome_desktop_item_unref (item);
+
+			chdir (curdir);
+			g_free (curdir);
 		} else {
-			gnome_execute_shell (NULL, swallow->path);
+			gnome_execute_shell (g_get_home_dir (), swallow->path);
 		}
 	}
 }
@@ -79,19 +99,26 @@ before_remove (Swallow *swallow)
 	if (swallow->clean_remove)
 		return TRUE;
 
-	dlg = gnome_message_box_new (_("A swallowed application appears to "
+	dlg = gtk_message_dialog_new (NULL /* parent */,
+				      0 /* flags */,
+				      GTK_MESSAGE_QUESTION,
+				      GTK_BUTTONS_NONE,
+				      _("A swallowed application appears to "
 				       "have died unexpectedly.\n"
-				       "Attempt to reload it?"),
-				     GNOME_MESSAGE_BOX_QUESTION,
-				     _("Reload"),
-				     GNOME_STOCK_BUTTON_CANCEL,
-				     NULL);
-	gnome_dialog_set_close (GNOME_DIALOG (dlg),
-				TRUE /* click_closes */);
+				       "Attempt to reload it?"));
+
+	gtk_dialog_add_button (GTK_DIALOG (dlg),
+			       GTK_STOCK_CANCEL,
+			       CANCEL_BUTTON);
+	gtk_dialog_add_button (GTK_DIALOG (dlg),
+			       _("Reload"),
+			       RELOAD_BUTTON);
+	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+					 RELOAD_BUTTON);
 	gtk_window_set_wmclass (GTK_WINDOW (dlg),
 				"swallow_crashed", "Panel");
 
-	if (gnome_dialog_run (GNOME_DIALOG (dlg)) == 0) {
+	if (gtk_dialog_run (GTK_DIALOG (dlg)) == 0) {
 
 		/* make the socket again */
 		swallow->socket = gtk_socket_new ();
@@ -102,13 +129,13 @@ before_remove (Swallow *swallow)
 			gtk_widget_set_usize(swallow->socket,
 					     swallow->width, swallow->height);
 
-		gtk_signal_connect_after (GTK_OBJECT (swallow->socket),
-					  "realize",
-					  GTK_SIGNAL_FUNC (socket_realized),
-					  NULL);
-		gtk_signal_connect (GTK_OBJECT (swallow->socket), "destroy",
-				    GTK_SIGNAL_FUNC (socket_destroyed),
-				    swallow);
+		g_signal_connect_after (G_OBJECT (swallow->socket),
+					"realize",
+					G_CALLBACK (socket_realized),
+					NULL);
+		g_signal_connect (G_OBJECT (swallow->socket), "destroy",
+				  G_CALLBACK (socket_destroyed),
+				  swallow);
 
 		gtk_container_add (GTK_CONTAINER (swallow->frame),
 				   swallow->socket);
@@ -169,7 +196,7 @@ socket_destroyed (GtkWidget *w, gpointer data)
 }
 
 static void
-really_add_swallow(GtkWidget *d,int button, gpointer data)
+really_add_swallow (GtkWidget *d, int response, gpointer data)
 {
 	GtkWidget *title_e = gtk_object_get_data(GTK_OBJECT(d), "title_e");
 	GtkWidget *exec_e = gtk_object_get_data(GTK_OBJECT(d), "exec_e");
@@ -180,12 +207,12 @@ really_add_swallow(GtkWidget *d,int button, gpointer data)
 	gboolean exactpos =
 		GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(d), "exactpos"));
 
-	switch (button) {
-	case 1: /* cancel */
-		gtk_widget_destroy(d);
+	switch (response) {
+	case CANCEL_BUTTON:
+		gtk_widget_destroy (d);
 		return;
-	case 2: /* help */
-		panel_show_help ("specialobjects.html#SWALLOWEDAPPS");
+	case HELP_BUTTON:
+		panel_show_help ("specialobjects", "SWALLOWEDAPPS");
 		return;
 	default:
 		break;
@@ -224,13 +251,18 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	GtkWidget *box, *i_box;
 	GtkAdjustment *adj;
 	
-	d = gnome_dialog_new(_("Create swallow applet"),
-			     GNOME_STOCK_BUTTON_OK,
-			     GNOME_STOCK_BUTTON_CANCEL,
-			     GNOME_STOCK_BUTTON_HELP,
-			     NULL);
-	gtk_window_set_wmclass(GTK_WINDOW(d),
-			       "create_swallow","Panel");
+	d = gtk_dialog_new_with_buttons (_("Create swallow applet"),
+					 NULL /* parent */,
+					 0 /* flags */,
+					 GTK_STOCK_CANCEL,
+					 CANCEL_BUTTON,
+					 GTK_STOCK_OK,
+					 OK_BUTTON,
+					 GTK_STOCK_HELP,
+					 HELP_BUTTON,
+					 NULL);
+	gtk_window_set_wmclass (GTK_WINDOW (d),
+				"create_swallow", "Panel");
 	/*gtk_window_set_position(GTK_WINDOW(d), GTK_WIN_POS_CENTER);*/
 	gtk_window_set_policy(GTK_WINDOW(d), FALSE, FALSE, TRUE);
 
@@ -239,27 +271,27 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	gtk_object_set_data(GTK_OBJECT(d),"exactpos",GINT_TO_POINTER(exactpos));
 
 	box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),box, TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),box, TRUE,TRUE,0);
 	w = gtk_label_new(_("Title of application to swallow"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
 	w = gnome_entry_new("swallow_title");
 	gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,0);
 	title_e = gnome_entry_gtk_entry(GNOME_ENTRY(w));
-	gtk_signal_connect(GTK_OBJECT(title_e),"activate",
-			   GTK_SIGNAL_FUNC(act_really_add_swallow),d);
+	g_signal_connect (G_OBJECT(title_e),"activate",
+			  G_CALLBACK(act_really_add_swallow),d);
 
 	box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),box, TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),box, TRUE,TRUE,0);
 	w = gtk_label_new(_("Command (optional)"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
 	w = gnome_file_entry_new("execute",_("Browse"));
 	gtk_box_pack_start(GTK_BOX(box),w,TRUE,TRUE,0);
 	exec_e = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (w));
-	gtk_signal_connect(GTK_OBJECT(exec_e),"activate",
-			   GTK_SIGNAL_FUNC(act_really_add_swallow),d);
+	g_signal_connect (G_OBJECT(exec_e),"activate",
+			  G_CALLBACK(act_really_add_swallow),d);
 
 	box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(d)->vbox),box, TRUE,TRUE,0);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox),box, TRUE,TRUE,0);
 
 	w = gtk_label_new(_("Width"));
 	gtk_box_pack_start(GTK_BOX(box),w,FALSE,FALSE,0);
@@ -283,17 +315,15 @@ ask_about_swallowing(PanelWidget *panel, int pos, gboolean exactpos)
 	height_s = gtk_spin_button_new(adj,0,0);
 	gtk_box_pack_start(GTK_BOX(box),height_s,FALSE,FALSE,0);
 	
-	gtk_signal_connect(GTK_OBJECT(d),"clicked",
-			   GTK_SIGNAL_FUNC(really_add_swallow),NULL);
+	g_signal_connect (G_OBJECT (d), "response",
+			  G_CALLBACK (really_add_swallow), NULL);
 	gtk_object_set_data(GTK_OBJECT(d),"title_e",title_e);
 	gtk_object_set_data(GTK_OBJECT(d),"exec_e",exec_e);
 	gtk_object_set_data(GTK_OBJECT(d),"width_s",width_s);
 	gtk_object_set_data(GTK_OBJECT(d),"height_s",height_s);
 
 
-	gnome_dialog_close_hides(GNOME_DIALOG(d),FALSE);
-
-	gnome_dialog_set_default(GNOME_DIALOG(d),0);
+	gtk_dialog_set_default_response (GTK_DIALOG (d), OK_BUTTON);
 
 	gtk_widget_grab_focus(title_e);
 
@@ -317,7 +347,7 @@ ignore_1st_click(GtkWidget *widget, GdkEvent *event)
 }
 
 static Swallow *
-create_swallow_applet(char *title, char *path, int width, int height, SwallowOrient orient)
+create_swallow_applet(const char *title, const char *path, int width, int height, SwallowOrient orient)
 {
 	Swallow *swallow;
 	
@@ -332,15 +362,15 @@ create_swallow_applet(char *title, char *path, int width, int height, SwallowOri
 	swallow->socket = gtk_socket_new ();
 	if (width != 0 || height != 0)
 		gtk_widget_set_usize(swallow->socket, width, height);
-	gtk_signal_connect_after (GTK_OBJECT (swallow->socket), "realize",
-				  GTK_SIGNAL_FUNC (socket_realized), NULL);
-	gtk_signal_connect (GTK_OBJECT (swallow->socket), "destroy",
-			    GTK_SIGNAL_FUNC (socket_destroyed), swallow);
+	g_signal_connect_after (G_OBJECT (swallow->socket), "realize",
+				G_CALLBACK (socket_realized), NULL);
+	g_signal_connect (G_OBJECT (swallow->socket), "destroy",
+			  G_CALLBACK (socket_destroyed), swallow);
 	
 	
 	swallow->handle_box = gtk_handle_box_new ();
-	gtk_signal_connect (GTK_OBJECT (swallow->handle_box), "event",
-			    GTK_SIGNAL_FUNC (ignore_1st_click), NULL);
+	g_signal_connect (G_OBJECT (swallow->handle_box), "event",
+			  G_CALLBACK (ignore_1st_click), NULL);
 	
 	gtk_container_add (GTK_CONTAINER(swallow->ebox),
 			   swallow->handle_box);
@@ -396,28 +426,26 @@ set_swallow_applet_orient(Swallow *swallow, SwallowOrient orient)
 }
 
 void
-load_swallow_applet (char *path, char *params, int width, int height,
+load_swallow_applet (const char *path, const char *params, int width, int height,
 		     PanelWidget *panel, int pos, gboolean exactpos)
 {
-	Swallow *swallow;
+	Swallow    *swallow;
+	AppletInfo *info;
 
 	swallow = create_swallow_applet (params, path, width, height,
 					 SWALLOW_HORIZONTAL);
 	if (swallow == NULL)
 		return;
 
-	if ( ! register_toy (swallow->ebox,
-			     ref_swallow (swallow),
-			     (GDestroyNotify) unref_swallow,
-			     panel, pos,
-			     exactpos, APPLET_SWALLOW))
+	info = panel_applet_register (swallow->ebox, ref_swallow (swallow),
+				      (GDestroyNotify) unref_swallow,
+				      panel, pos, exactpos, APPLET_SWALLOW);
+	if (!info)
 		return;
 
 	swallow->clean_remove = FALSE;
 
-	applet_add_callback (applets_last->data, "help",
-			     GNOME_STOCK_PIXMAP_HELP,
-			     _("Help"));
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("Help"));
 
 	swallow_launch (swallow);
 }
